import logging
from datetime import datetime

def createFilename()->str:
    '''create a unique filename from the given parameters and time'''
    return(datetime.now().strftime('%d%m%y%H%M%S') + ".png")

def getLogger(name: str, level: str) -> logging.Logger:
    '''create a new logging object'''
    logging.basicConfig(level=level)
    return(logging.getLogger(name))


import math
import re
from typing import Tuple


class UtfColor:
    def __init__(self):
        self.encoding = {
            "e": "00",#0
            "n": "ff",#255
            "i": "80",#128
            "s": "40",#64
            "r": "c0",#192
            "a": "20",#32
            "t": "e0",#224
            "d": "60",#96
            "h": "a0",#160
            "u": "10",#16
            "l": "f0",#240
            "c": "30",#48
            "g": "d0",#208
            "m": "50",#80
            "o": "b0",#176
            "b": "70",#112
            "w": "90",#144
            "f": "08",#8
            "k": "f8",#248
            "z": "18",#24
            "p": "e8",#232
            "v": "28",#40
            "j": "d8",#216
            "y": "38",#56
            "x": "c8",#200
            "q": "48",#72
            "1": "b8",#184
            "2": "58",#88
            "3": "a8",#168
            "4": "68",#104
            "5": "98",#152
            "6": "78",#120
            "7": "88",#136
            "8": "04",#4
            "9": "fb",#251
            "0": "0c",#12
            ".": "f4",#244
            ",": "14",#20
            " ": "ec",#236
        }

    def encode(self, txt:str)->str:
        '''encodes text to utf-color'''
        encodedText = ""
        for c in txt:
            encodedText += self.encoding[c]
        return(encodedText)

    def prepareText(self, txt:str)->str:
        '''prepare the input text for usage of utf-color. This will remove all chars that are unsupported by utf-color'''
        #convert capital letters to lower case
        txt = txt.lower()
        #remove unsupported characters
        txt = re.sub("[^a-z0-9,. ]", "", txt)
        #return result
        return(txt)

    def adjustTextLen(self, txt:str, x:int, y:int)->str:
        '''fill the last column with whitespaces (+16 extra whitespaces to avoid errors)'''
        return(txt.ljust((x*y*3)+16))

    def getSize(self, n:int)->Tuple[int,int]:
        '''calculate the size of the resulting image'''
        #divide n by 3 because every pixel will represent 3 chars
        n = n/3
        #calculate the x-len
        x = int(math.sqrt(n))
        #calculate the y-len
        y = n / x
        #if is not a float with a 0 after the dot, then add an extra column of pixels, so every char will be in the image
        if y - int(y) > 0.0: x += 1
        #return result
        return(x, int(y))


from time import perf_counter

from utfcolor.main import txtToImg
from utfcolor.argparams import parseArgs
import utfcolor.util as util


startTime = perf_counter()

args = parseArgs()

#get the text that will be converted into an image
text = args.pop("text")
if not text:
    with open(args.pop("file")) as f:
        text = f.read()

#start converting the text
txtToImg(text=text, encoding=args.pop("encoding"), outputFile=args.pop("output"), scale=args.pop("scale"))

if args.pop("performancecounter"):
    endTime = perf_counter()
    logger = util.getLogger("utfcolor", "INFO")
    logger.info(f'New Image created in {endTime - startTime: 0.2f} second(s).')


from PIL import Image, ImageDraw

def upScale(img:Image, scale:int)->Image:
    '''Upscaling of an image. Borders between pixels will not become soft.'''
    x,y = img.size  #get the size of the given image
    newImg = Image.new(mode="RGB", size=(x*scale, y*scale))     #create a new image with the correct size
    draw = ImageDraw.Draw(newImg)

    #iterate over all pixels in the old image
    for px in range(0, x*scale, scale):
        for py in range(0, y*scale, scale):
            #get color-value of the pixel
            color = img.getpixel((px/scale,py/scale))
            #multiply the pixel
            for sx in range(scale):
                for sy in range(scale):
                    draw.point((px+sx,py+sy), fill=color)    #put pixel into image

    #return the result
    return(newImg)



from utfcolor import utfcolorv1

class StandardEncoding(utfcolorv1.UtfColor):
    def __init__(self, encoding:str="utf-8"):
        self.encoding = encoding

    def encode(self, txt:str)->str:
        '''encodes text'''
        txt = txt.encode(self.encoding)
        return(txt.hex())

    def prepareText(self, txt:str)->str:
        '''NOOP (no text preparation for standard encodings)'''
        return(txt)


import random

from utfcolor import utfcolorv2


class UtfColorCustom(utfcolorv2.UtfColorV2):
    def __init__(self, space:int=40):
        self.space = space
        self.encoding = self.createEncoding()

    def createEncoding(self)->dict:
        encoding = {}
        letters = {
            "e": "",
            "n": "",
            "i": "",
            "s": "",
            "r": "",
            "a": "",
            "t": "",
            "d": "",
            "h": "",
            "u": "",
            "l": "",
            "c": "",
            "g": "",
            "m": "",
            "o": "",
            "b": "",
            "w": "",
            "f": "",
            "k": "",
            "z": "",
            "p": "",
            "v": "",
            "j": "",
            "y": "",
            "x": "",
            "q": ""
        }
        numbers = {
            "1": "",
            "2": "",
            "3": "",
            "4": "",
            "5": "",
            "6": "",
            "7": "",
            "8": "",
            "9": "",
            "0": ""
        }
        special = {
            ".": "",
            ",": "",
            " ": ""
        }

        l = len(letters) + len(numbers) + len(special)
        start = random.randint(0, 16777215 - l * self.space)

        def addChars(chars:dict):
            for n,c in enumerate(chars):
                #encoding[c] = hex(start + n)[2:]
                encoding[c] = hex(start + n*self.space)[2:]
        
        addChars(letters)
        
        start = random.randint(0, 16777215 - l)
        addChars(numbers)
        
        start = random.randint(0, 16777215 - l)
        addChars(special)

        return(encoding)


import argparse

def parseArgs() -> dict:
    parser = argparse.ArgumentParser(description="Convert any text to an image.")

    parser.add_argument(
    "-f",
    "--file",
    type=str,
    help="Input text file path.")

    parser.add_argument(
    "-t",
    "--text",
    type=str,
    help="Directly input text for encoding.")

    parser.add_argument(
    "-e",
    "--encoding",
    type=str,
    default="utf-color",
    help="Encoding to use.")

    parser.add_argument(
    "-o",
    "--output",
    type=str,
    default="",
    help="Name of the output file.")

    parser.add_argument(
    "-s",
    "--scale",
    type=int,
    default=0,
    help="Upscaling of the resulting image.")

    parser.add_argument(
    "-pc",
    "--performancecounter",
    action='store_true', 
    default=False,
    help="Count the seconds until the image is created.")

    arguments = parser.parse_args()

    return {
        "file": arguments.file,
        "text": arguments.text,
        "encoding": arguments.encoding,
        "output": arguments.output,
        "scale": arguments.scale,
        "performancecounter": arguments.performancecounter
    }


import setuptools

with open("requirements.txt") as fh:
    requirements = fh.read().splitlines()

setuptools.setup(
    name="utfcolor",
    version="1.0.0",
    license="MIT",
    author="5omega",
    author_email="5omega@posteo.de",
    description="Convert text to images with Python",
    long_description_content_type="text/plain",
    long_description="Visit https://github.com/5omega/utfcolor for more information.",
    url="https://github.com/5omega/utfcolor",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    install_requires=requirements,
    python_requires=">=3.6.9",
)



from PIL import Image, ImageDraw

from utfcolor.utfcolorv1 import UtfColor
from utfcolor.utfcolorv2 import UtfColorV2
from utfcolor.utfcolorv3 import UtfColorV3
from utfcolor.createEncoding import UtfColorCustom
from utfcolor.standardEncoding import StandardEncoding
from utfcolor.upScale import upScale
import utfcolor.util as util


def txtToImg(text:str, encoding:str="utf-color", outputFile:str="", scale:int=0) -> None:
    '''convert any text to an image'''

    #create encoding objects
    if encoding == "utf-color":
        encoder = UtfColor()
    elif encoding == "utf-colorv2":
        encoder = UtfColorV2()
    elif encoding == "utf-colorv3":
        encoder = UtfColorV3()
    elif encoding == "rng-utf-color":
        encoder = UtfColorCustom()
    else:
        encoder = StandardEncoding(encoding)

    #prepare text for encoding
    text = encoder.prepareText(text)
    x,y = encoder.getSize(len(text))
    text = encoder.adjustTextLen(text,x,y)
    #encode text
    text = encoder.encode(text)

    #create a new image
    img = Image.new(mode="RGB", size=(x,y))
    draw = ImageDraw.Draw(img)

    #fill the image with pixels
    for px in range(x):
        for py in range(y):
            draw.point((px,py), fill=f"#{text[:6]}")    #put pixel into image
            #img.paste(((f"#{text[:6]}")), (px,py,x,y))
            text = text[6:]     #remove hex value from text

    #save the image
    if outputFile == "": outputFile = util.createFilename()
    if scale != 0: img = upScale(img, scale)
    img.save(outputFile)


import math
from typing import Tuple

import utfcolor.utfcolorv1 as utfcolorv1


class UtfColorV2(utfcolorv1.UtfColor):
    def __init__(self):
        self.encoding = {
            "e": "FFFF00",#Yellow
            "n": "32CD32",#Green
            "i": "FFA500",#Orange
            "s": "008080",#Blue/Teal
            "r": "DC143C",#Red/Crimson
            "a": "4B0082",#Purple/Indigo
            "t": "6B8E23",#Green
            "d": "6A5ACD",#SlateBlue
            "h": "FF69B4",#HotPink
            "u": "006400",#DarkGreen
            "l": "B8860B",#DarkGoldenRod
            "c": "F0E68C",#Yellow
            "g": "0000FF",#Blue
            "m": "FFE4E1",#MistyRose
            "o": "800080",#Purple
            "b": "8B4513",#Brown/SaddleBrown
            "w": "FA8072",#Red/Salmon
            "f": "ADFF2F",#GreenYellow
            "k": "00CED1",#Blue/DarkTurquoise
            "z": "C71585",#MediumVioletRed
            "p": "FF4500",#OrangeRed
            "v": "BC8F8F",#RosyBrown
            "j": "FFD700",#Yellow
            "y": "000080",#Blue/Navy
            "x": "FF00FF",#Magenta
            "q": "00FFFF",#Blue/Aqua
            "1": "FF0000",#Red
            "2": "2F4F4F",#DarkSlateGray
            "3": "CD853F",#Peru
            "4": "9ACD32",#YellowGreen
            "5": "B22222",#FireBrick
            "6": "00FF7F",#SpringGreen
            "7": "87CEFA",#Blue/LightSkyBlue 
            "8": "FFDAB9",#PeachPuff
            "9": "90EE90",#LightGreen
            "0": "4682B4",#SteelBlue
            ".": "ffffff",#White
            ",": "FFDEAD",#NavajoWhite
            " ": "000000",#Black
        }

    def getSize(self, n:int)->Tuple[int,int]:
        '''calculate the size of the resulting image'''
        #calculate the x-len
        x = int(math.sqrt(n))
        #calculate the y-len
        y = n / x
        #if is not a float with a 0 after the dot, then add an extra column of pixels, so every char will be in the image
        if y - int(y) > 0.0: x += 1
        #return result
        return(x, int(y))

    def adjustTextLen(self, txt:str, x:int, y:int)->str:
        '''fill the last column with whitespaces (+16 extra whitespaces to avoid errors)'''
        return(txt.ljust((x*y)+16))


import utfcolor.utfcolorv2 as utfcolorv2


class UtfColorV3(utfcolorv2.UtfColorV2):
    def __init__(self):
        self.encoding = {
            "e": "00FA9A",#MediumSpringGreen
            "n": "191970",#MidnightBlue
            "i": "3CB371",#MediumSeaGreen
            "s": "0000FF",#Blue
            "r": "00008B",#DarkBlue
            "a": "32CD32",#LimeGreen
            "t": "0000CD",#MediumBlue
            "d": "4169E1",#RoyalBlue
            "h": "1E90FF",#DodgerBlue
            "u": "008000",#Green
            "l": "00BFFF",#DeepSkyBlue
            "c": "6495ED",#CornflowerBlue
            "g": "87CEEB",#SkyBlue
            "m": "87CEFA",#LightSkyBlue
            "o": "9ACD32",#YellowGreen
            "b": "B0E0E6",#PowderBlue
            "w": "B0C4DE",#LightSteelBlue
            "f": "4682B4",#SteelBlue
            "k": "5F9EA0",#CadetBlue
            "z": "00CED1",#DarkTurquoise
            "p": "7FFFD4",#Aquamarine
            "v": "AFEEEE",#PaleTurquoise
            "j": "00FFFF",#Cyan
            "y": "008080",#Teal
            "x": "20B2AA",#LightSeaGreen
            "q": "66CDAA",#MediumAquaMarine
            "1": "FF69B4",#HotPink
            "2": "C71585",#MediumVioletRed
            "3": "9400D3",#DarkViolet
            "4": "DA70D6",#Orchid
            "5": "9370DB",#MediumPurple
            "6": "6A5ACD",#SlateBlue
            "7": "663399",#RebeccaPurple
            "8": "4B0082",#Indigo
            "9": "FFB6C1",#LightPink
            "0": "DB7093",#PaleVioletRed
            ".": "000000",#Black
            ",": "A9A9A9",#DarkGray
            " ": "FFFFFF"#White
        }